#include <windows.h>
#include "RAMpageReg.h"

static char RunSubKeyStr[] = "Software\\Microsoft\\Windows\\CurrentVersion\\Run";
static char RunValueName[] = "RAMpage";

static char SubKeyStrV13[] = "Software\\" REG_COMPANY_NAME "\\" REG_APP_NAME "\\1.3";
static char SubKeyStrSection[] = "Software\\" REG_COMPANY_NAME "\\" REG_APP_NAME "\\" REG_SECTION_NAME;
static char SubKeyStrApp[] = "Software\\" REG_COMPANY_NAME "\\" REG_APP_NAME;
static char SubKeyStrCompany[] = "Software\\" REG_COMPANY_NAME;

//
// Called as part of the uninstall process to remove all traces of RAMpage
//
void RemoveRAMpage()
{
	StopRAMpage(50);
	DeleteAutoStartRegistryValue();
	//
	// Remove HKEY_CURRENT_USER//Software/RAMpage from the bottom up
	//
	RegDeleteKey(HKEY_CURRENT_USER,	SubKeyStrV13);
	RegDeleteKey(HKEY_CURRENT_USER,	SubKeyStrSection);
	RegDeleteKey(HKEY_CURRENT_USER,	SubKeyStrApp);
	RegDeleteKey(HKEY_CURRENT_USER,	SubKeyStrCompany);
}

void StopRAMpage(unsigned int PauseMilliseconds)
{
	HWND hWnd;			// Handle of any running RAMpage Window
	LRESULT msgr;		// Message Result
	int attempts = 0;

	//
	// Look for RAMpage windows and ask them to stop running
	//
	hWnd = FindWindow (NULL,"RAMpage");
	while ((hWnd != NULL) && (attempts < 10))
	{
		msgr = SendMessage(
			hWnd,	// handle of destination window
			WM_CLOSE,	// message to send
			(WPARAM) 0,	// first message parameter
			(LPARAM) 0	// second message parameter
			);
		Sleep(PauseMilliseconds); // Give the app time to stop before continuing
		attempts++;
		hWnd = FindWindow (NULL,"RAMpage");
	}
}	

void AddAutoStartRegistryValue(const LPSTR lpszParams, BOOL RunHidden) 
{
	HKEY hKey;
	char ExecSpec[_MAX_PATH];
	char CurrDir[_MAX_PATH];

	if (!GetCurrentDirectory(_MAX_PATH-17-strlen(lpszParams), CurrDir))
	{
		MessageBox(NULL,"Couldn't get current directory!", "RAMpage", MB_ICONEXCLAMATION|MB_OK);
		return;
	}
	strcpy(ExecSpec,"\"");
	strcat(ExecSpec,CurrDir);
	strcat(ExecSpec,"\\");
	if (RunHidden)
		strcat(ExecSpec,REG_HIDDEN_PROGNAME);
	else
		strcat(ExecSpec,REG_NORMAL_PROGNAME);
	strcat(ExecSpec,"\"");
	strcat(ExecSpec,lpszParams);

	if (RegOpenKeyEx(
		HKEY_LOCAL_MACHINE,	// handle of open key 
		RunSubKeyStr,	// address of name of subkey to delete
		0,
		KEY_SET_VALUE,
		&hKey
		) != ERROR_SUCCESS)
		MessageBox(NULL,
			"ERROR: Can't Open the \"Run\" Registry Key!\n"
			"(RAMpage will not start when Windows starts.)", 
			"RAMpage", MB_ICONEXCLAMATION|MB_OK);
	else
	{
		if (RegSetValueEx(
			hKey,	// handle of key to set value for  
			RunValueName,	// address of value to set 
			0,	// reserved 
			REG_SZ,	// flag for value type 
			ExecSpec,	// address of value data 
			strlen(ExecSpec)+1 	// size of value data 
			) != ERROR_SUCCESS)
			MessageBox(NULL,"Couldn't set \"Run\" value!", "RAMpage", MB_ICONEXCLAMATION|MB_OK);
	}
}

void DeleteAutoStartRegistryValue() 
{
	HKEY hKey;			// Handle of Registry Key

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,	RunSubKeyStr, 0, KEY_SET_VALUE, &hKey) == ERROR_SUCCESS)
		RegDeleteValue(hKey, RunValueName);
	//
	// Note: Try to delete the value for HKEY_CURRENT_USER as well.
	// (Just in case it was manually added here on Win95 machines.)
	//
	if (RegOpenKeyEx(HKEY_CURRENT_USER,	RunSubKeyStr, 0, KEY_SET_VALUE, &hKey) == ERROR_SUCCESS)
		RegDeleteValue(hKey, RunValueName);
}

void DeleteRegistryValues() 
{
	DeleteAutoStartRegistryValue();

	// Get rid of the installer's useless version info
	RegDeleteKey(HKEY_CURRENT_USER,	SubKeyStrV13);

	if (RegDeleteKey(
		HKEY_CURRENT_USER,	// handle of open key 
		SubKeyStrSection	// address of name of subkey to delete
		) != ERROR_SUCCESS)
		MessageBox(NULL,"Couldn't Delete Section Key!", "RAMpage", MB_ICONEXCLAMATION|MB_OK);
	else
	{
		if (RegDeleteKey(
			HKEY_CURRENT_USER,	// handle of open key 
			SubKeyStrApp		// address of name of subkey to delete
			) != ERROR_SUCCESS)
			MessageBox(NULL, "Couldn't Delete Application Key!", "RAMpage", MB_ICONEXCLAMATION|MB_OK);
		else
		{
			if (RegDeleteKey(
				HKEY_CURRENT_USER,	// handle of open key 
				SubKeyStrCompany	// address of name of subkey to delete
				) != ERROR_SUCCESS)
				MessageBox(NULL, "Couldn't Delete Company Key!", "RAMpage", MB_ICONEXCLAMATION|MB_OK);
		}
	}
}

